//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;

import junit.framework.*;
import junit.textui.TestRunner;

public final class CardComparatorTest extends TestCase {
	public CardComparatorTest(String name) {
		super(name);
	}

	//
	// Test CardComparator().
	//
	public void testCostructor1() {
		try {
			CardComparator cc = new CardComparator();
			assertEquals(cc.getMode(),
				CardComparisonMode.PRICE_THEN_EXPANSION);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test CardComparator(mode) and getMode().
	//
	public void testCostructor2() {
		try {
			CardComparator cc1
				= new CardComparator(CardComparisonMode.PRICE_THEN_EXPANSION);
			CardComparator cc2
				= new CardComparator(CardComparisonMode.EXPANSION_THEN_PRICE);

			assertEquals(cc1.getMode(),
				CardComparisonMode.PRICE_THEN_EXPANSION);
			assertEquals(cc2.getMode(),
				CardComparisonMode.EXPANSION_THEN_PRICE);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test Compare() for both PRICE_THEN_EXPANSION and EXPANSION_THEN_PRICE
	// modes.
	//
	private void testCmpareCommon(CardComparator cc) {
		try {
			Expansion x = new Expansion("", "", 1000);
			Card c1;
			Card c2;

			//
			// "1" <=> "1"
			//
			c1 = new Card("", "", "1", x);
			c2 = new Card("", "", "1", x);
			assertTrue(cc.compare(c1, c2) == 0);

			//
			// "9" <=> "10"
			//
			c1 = new Card("", "", "10", x);
			c2 = new Card("", "", "9", x);
			assertTrue(cc.compare(c1, c2) > 0);

			c1 = new Card("", "", "9", x);
			c2 = new Card("", "", "10", x);
			assertTrue(cc.compare(c1, c2) < 0);

			//
			// "9+" <=> "10+"
			//
			c1 = new Card("", "", "10+", x);
			c2 = new Card("", "", "9+", x);
			assertTrue(cc.compare(c1, c2) > 0);

			c1 = new Card("", "", "9+", x);
			c2 = new Card("", "", "10+", x);
			assertTrue(cc.compare(c1, c2) < 0);

			//
			// "9x" <=> "10x"
			//
			c1 = new Card("", "", "10x", x);
			c2 = new Card("", "", "9x", x);
			assertTrue(cc.compare(c1, c2) > 0);

			c1 = new Card("", "", "9x", x);
			c2 = new Card("", "", "10x", x);
			assertTrue(cc.compare(c1, c2) < 0);

			//
			// "9+" <=> "10*"
			//
			c1 = new Card("", "", "10*", x);
			c2 = new Card("", "", "9+", x);
			assertTrue(cc.compare(c1, c2) > 0);

			c1 = new Card("", "", "9+", x);
			c2 = new Card("", "", "10*", x);
			assertTrue(cc.compare(c1, c2) < 0);

			//
			// "9y" <=> "10x"
			//
			c1 = new Card("", "", "10x", x);
			c2 = new Card("", "", "9y", x);
			assertTrue(cc.compare(c1, c2) > 0);

			c1 = new Card("", "", "9y", x);
			c2 = new Card("", "", "10x", x);
			assertTrue(cc.compare(c1, c2) < 0);

			//
			// "1+" <=> "1*"
			//
			c1 = new Card("", "", "1+", x);
			c2 = new Card("", "", "1*", x);
			assertTrue(cc.compare(c1, c2) > 0);

			c1 = new Card("", "", "1*", x);
			c2 = new Card("", "", "1+", x);
			assertTrue(cc.compare(c1, c2) < 0);

			//
			// "+9" <=> "+10"
			//
			c1 = new Card("", "", "+10", x);
			c2 = new Card("", "", "+9", x);
			assertTrue(cc.compare(c1, c2) > 0);

			c1 = new Card("", "", "+9", x);
			c2 = new Card("", "", "+10", x);
			assertTrue(cc.compare(c1, c2) < 0);

			//
			// "1" <=> "?"
			//
			c1 = new Card("", "", "?", x);
			c2 = new Card("", "", "1", x);
			assertTrue(cc.compare(c1, c2) > 0);

			c1 = new Card("", "", "1", x);
			c2 = new Card("", "", "?", x);
			assertTrue(cc.compare(c1, c2) < 0);

			//
			// "?" <=> "?"
			//
			c1 = new Card("", "", "?", x);
			c2 = new Card("", "", "?", x);
			assertTrue(cc.compare(c1, c2) == 0);

			//
			// "*" <=> "?"
			//
			c1 = new Card("", "", "?", x);
			c2 = new Card("", "", "*", x);
			assertTrue(cc.compare(c1, c2) > 0);

			c1 = new Card("", "", "*", x);
			c2 = new Card("", "", "?", x);
			assertTrue(cc.compare(c1, c2) < 0);

		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test Compare() in PRICE_THEN_EXPANSION mode.
	//
	public void testCmpare1() {
		try {
			CardComparator cc
				= new CardComparator(CardComparisonMode.PRICE_THEN_EXPANSION);

			//
			// Test cases independent from mode.
			//
			testCmpareCommon(cc);

			//
			// Same price, different expansion.
			//
			Expansion x1 = new Expansion("", "", 1000);
			Expansion x2 = new Expansion("", "", 2000);
			Card c1;
			Card c2;

			c1 = new Card("", "", "1", x2);
			c2 = new Card("", "", "1", x1);
			assertTrue(cc.compare(c1, c2) > 0);

			c1 = new Card("", "", "1", x1);
			c2 = new Card("", "", "1", x2);
			assertTrue(cc.compare(c1, c2) < 0);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test Compare() in EXPANSION_THEN_PRICE mode.
	//
	public void testCmpare2() {
		try {
			CardComparator cc
				= new CardComparator(CardComparisonMode.EXPANSION_THEN_PRICE);

			//
			// Test cases independent from mode.
			//
			testCmpareCommon(cc);

			//
			// Same expansion, different price.
			//
			Expansion x = new Expansion("", "", 1000);
			Card c1;
			Card c2;

			c1 = new Card("", "", "2", x);
			c2 = new Card("", "", "1", x);
			assertTrue(cc.compare(c1, c2) > 0);

			c1 = new Card("", "", "1", x);
			c2 = new Card("", "", "2", x);
			assertTrue(cc.compare(c1, c2) < 0);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test equals().
	//
	public void testEquals() {
		try {
			CardComparator cc1
				= new CardComparator();
			CardComparator cc2
				= new CardComparator(CardComparisonMode.PRICE_THEN_EXPANSION);
			CardComparator cc3
				= new CardComparator(CardComparisonMode.EXPANSION_THEN_PRICE);

			assertTrue(cc1.equals(cc1));
			assertTrue(cc1.equals(cc2));
			assertFalse(cc1.equals(cc3));
			assertFalse(cc1.equals(CardComparisonMode.PRICE_THEN_EXPANSION));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test toString().
	//
	public void testToString() {
		try {
			CardComparator cc1
				= new CardComparator(CardComparisonMode.PRICE_THEN_EXPANSION);
			CardComparator cc2
				= new CardComparator(CardComparisonMode.EXPANSION_THEN_PRICE);

			String str1 = cc1.toString();
			String str2 = cc2.toString();

			assertTrue(str1.equals("CardComparator (PRICE_THEN_EXPANSION)"));
			assertTrue(str2.equals("CardComparator (EXPANSION_THEN_PRICE)"));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(CardComparatorTest.class);
	}
}
