//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;

import junit.framework.*;
import junit.textui.TestRunner;

public final class CardTest extends TestCase {
	public CardTest(String name) {
		super(name);
	}

	//
	// Test Card(id, title, price, expansion).
	//
	public void testCostructor() {
		try {
			Expansion x = new Expansion("test-expansion-id", "");
			String id = "test-card-id";
			String title = "test-card-title";
			String price = "test-card-price";

			Card c = new Card(id, title, price, x);

			assertEquals(c.getId(), id);
			assertEquals(c.getTitle(), title);
			assertEquals(c.getPrice(), price);
			assertEquals(c.getExpansion(), x);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test Card(card).
	//
	public void testCopyCostructor() {
		try {
			Expansion x = new Expansion("test-expansion-id", "");
			String id = "test-card-id";
			String title = "test-card-title";
			String price = "test-card-price";

			Card c1 = new Card(id, title, price, x);
			Card c2 = new Card(c1);

			assertEquals(c2.getId(), id);
			assertEquals(c2.getTitle(), title);
			assertEquals(c2.getPrice(), price);
			assertEquals(c2.getExpansion(), x);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test getId(), getTitle(), getPrice() and getExpansion().
	//
	public void testGetters() {
		try {
			int expansionsSize = 10;
			int cardsSize = 1000;
			String id = "test-card-id";
			String title = "test-card-title";
			String price = "test-card-price";

			Expansion[] expansions = new Expansion[expansionsSize];
			for (int i = 0; i < expansionsSize; i++)
				expansions[i] = new Expansion("test-expansion-id" + i, "");

			Card[] cards = new Card[cardsSize];
			for (int i = 0; i < cardsSize; i++) {
				cards[i] = new Card(id + i, title + i, price + i,
					expansions[i % expansionsSize]);
			}

			for (int i = 0; i < cardsSize; i++) {
				assertEquals(cards[i].getId(), id + i);
				assertEquals(cards[i].getTitle(), title + i);
				assertEquals(cards[i].getPrice(), price + i);
				assertEquals(cards[i].getExpansion(),
					expansions[i % expansionsSize]);
			}
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test equals().
	//
	public void testEquals() {
		try {
			Expansion x = new Expansion("test-expansion-id", "");
			Card c1 = new Card("a", "a", "a", x);
			Card c2 = new Card("a", "b", "b", x);
			Card c3 = new Card("b", "a", "a", x);
			String s = "a";

			assertTrue(c1.equals(c1));
			assertTrue(c1.equals(c2));
			assertFalse(c1.equals(c3));
			assertFalse(c1.equals(s));
			assertFalse(c1.equals(null));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test compareTo().
	//
	public void testCompareTo() {
		try {
			Expansion x = new Expansion("test-expansion-id", "");
			Card c1 = new Card("b", "a", "a", x);
			Card c2 = new Card("a", "a", "a", x);
			Card c3 = new Card("c", "a", "a", x);
			Card c4 = new Card("b", "a", "a", x);

			assertTrue(c1.compareTo(c1) == 0);
			assertTrue(c1.compareTo(c2) >  0);
			assertTrue(c1.compareTo(c3) <  0);
			assertTrue(c1.compareTo(c4) == 0);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test toString().
	//
	public void testToString() {
		try {
			Expansion x = new Expansion("test-expansion-id", "");
			String id = "test-card-id";
			String title = "test-card-title";
			String price = "test-card-price";

			Card c = new Card(id, title, price, x);

			assertEquals(c.toString(), title);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(CardTest.class);
	}
}
