//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;

import junit.framework.*;
import junit.textui.TestRunner;

public final class ExpansionTest extends TestCase {
	public ExpansionTest(String name) {
		super(name);
	}

	//
	// Test Expansion(id, title).
	//
	public void testCostructor1() {
		try {
			String id = "test-id";
			String title = "test-title";
			Expansion x = new Expansion(id, title);

			assertEquals(x.getId(), id);
			assertEquals(x.getTitle(), title);
			assertEquals(x.getPreference(), Integer.MAX_VALUE);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test Expansion(id, title, preference).
	//
	public void testCostructor2() {
		try {
			String id = "test-id";
			String title = "test-title";
			int preference = 1000;
			Expansion x = new Expansion(id, title, preference);

			assertEquals(x.getId(), id);
			assertEquals(x.getTitle(), title);
			assertEquals(x.getPreference(), preference);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test Expansion(expansion).
	//
	public void testCopyCostructor() {
		try {
			String id = "test-id";
			String title = "test-title";
			int preference = 1000;

			Expansion x1 = new Expansion(id, title, preference);
			Expansion x2 = new Expansion(x1);

			assertEquals(x2.getId(), id);
			assertEquals(x2.getTitle(), title);
			assertEquals(x2.getPreference(), preference);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test getId(), getTitle() and getPreference().
	//
	public void testGetters() {
		try {
			int expansionsSize = 100;

			String id = "test-id";
			String title = "test-title";
			int preference = 1000;
			Expansion[] expansions = new Expansion[expansionsSize];
			for (int i = 0; i < expansionsSize; i++) {
				expansions[i] = new Expansion("test-id" + i, title + i,
					preference + i);
			}

			for (int i = 0; i < expansionsSize; i++) {
				assertEquals(expansions[i].getId(), id + i);
				assertEquals(expansions[i].getTitle(), title + i);
				assertEquals(expansions[i].getPreference(), preference + i);
			}
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test equals(object).
	//
	public void testEquals() {
		try {
			Expansion x1 = new Expansion("a", "a", 0);
			Expansion x2 = new Expansion("a", "b", 0);
			Expansion x3 = new Expansion("b", "a", 0);
			String s = "a";

			assertTrue(x1.equals(x1));
			assertTrue(x1.equals(x2));
			assertFalse(x1.equals(x3));
			assertFalse(x1.equals(s));
			assertFalse(x1.equals(null));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test compareTo(expansion).
	//
	public void testCompareTo() {
		try {
			Expansion x1 = new Expansion("b", "a", 0);
			Expansion x2 = new Expansion("a", "a", 0);
			Expansion x3 = new Expansion("c", "a", 0);
			Expansion x4 = new Expansion("b", "a", 0);

			assertTrue(x1.compareTo(x1) == 0);
			assertTrue(x1.compareTo(x2) >  0);
			assertTrue(x1.compareTo(x3) <  0);
			assertTrue(x1.compareTo(x4) == 0);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test toString().
	//
	public void testToString() {
		try {
			String id = "test-id";
			String title = "test-title";
			Expansion x = new Expansion(id, title);

			assertEquals(x.toString(), title);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(ExpansionTest.class);
	}
}
